asm_TPA_YM<-function(L=NULL,CA=NULL,sMax=NULL,dataFile=NULL,StrainBounds=c(0.075,0.15)){
  
  if(is.null(L)|| is.null(CA) || is.null(sMax) || is.null(dataFile)){print("Inadequate input");}
  
  database=read.csv(dataFile,header=FALSE,skip=4)
  names(database)=c("Force[N]","Distance[m]","Time[s]");
  
  ld = dim(database)[1]
  
  #User Inputs:
  L1 = L
  CA1 = CA; 
  
  .pardefault <- par(no.readonly = TRUE)
  # ----------------------------------------------------------------------------
  # Preliminary Plotting and Exploration:
  par(mar=c(5, 4, 4, 6) + 0.1)

  plot(database$`Time[s]`,database$`Force[N]`,axes=FALSE,xlab=" ",ylab = " ",type="l",lty=2);
  axis(2, ylim=c(0,11),col="black",las=1)  ## las=1 makes horizontal labels
  mtext("force [N]",side=2,line=2.5)
  box()
  # 
  par(new=TRUE)
  # 
  plot(database[,3],database[,2], ylim=c(0,10),axes=FALSE,xlab=" ",ylab=" ",type="l",lty=2, col="red")
  mtext("compression [mm]",side=4,col="red",line=4,las=3)
  axis(4, ylim=c(0,10), col="red",col.axis="red",las=1)

  axis(1,pretty(range(database[,3]),10))
  mtext("time (s)",side=1,col="black",line=2.5)
  abline(h=0.15*L1,col="blue",lty=4)

  dev.copy(pdf,"asm-TPA.pdf")
  dev.off()

  par(.pardefault)
  
  
  # Real Work
  strain = array(0,dim=c(ld,1));
  stress = array(0,dim=c(ld,1));
  
  
  
  for(i in 1:ld){
    strain[i] = database[i,2]/(L1);
    stress[i] = database[i,1]/(CA1);
  }
  
  a = which(strain[,1]==sMax);
  amin = min(a);
  amax = max(a);
  ascent1 = c(1:amin);
  la1 = length(ascent1)
  d1 = amin+1;
  d2 = amin+la1-1;
  descent1 = c(d1:d2) 
  
  b = which(strain[,1]==0);
  bmax = max(b);
  ascent2=c(bmax:amax);
  d3 = amax+1;
  descent2 = c(d3:ld);
  
  # tiff("Stress-Strain-Curve.tiff", width = 6, height = 6, units = 'in', res = 300)
  plot(strain[ascent1,1],stress[ascent1,1],type="l",lty=1,col="black",xlab="strain [mm/mm]",ylab=expression(paste("stress [",N/mm^2,"]")));
  matplot(strain[descent1,1],stress[descent1,1],type="l",lty=1,col="red",add=TRUE)
  matplot(strain[ascent2,1],stress[ascent2,1],type="l",lty=2,col="black",add=TRUE)
  matplot(strain[descent2,1],stress[descent2,1],type="l",lty=2,col="red",add=TRUE)
  dev.copy(pdf,"asm-StressStrain.pdf")
  dev.off()
  
  
  Hardness1 = max(database[c(ascent1,descent1),1]);
  Distance1 = max(database[which(database[c(ascent1,descent1),1]==Hardness1),3]);

  Hardness2 = max(database[c(ascent2,descent2),1]);
  Distance2 = max(database[which(database[,1]==Hardness2),3] - database[ascent2[min(which(database[ascent2,1]>0))],3]);

  trapz<-function(x,y){
    a = 2:length(x)
    z = as.double( (x[a] - x[a-1]) %*% (y[a] + y[a-1])) / 2;
    return(z)
  }

  x = database[c(ascent1,descent1),3];
  y = database[c(ascent1,descent1),1];
  Area1 = trapz(x,y)

  x = database[c(ascent2,descent2),3];
  y = database[c(ascent2,descent2),1];
  Area2 = trapz(x,y)

  pH = max(which(database[,1]==Hardness1));
  if(length(pH)!=0){
  x = database[c(1:pH),3];
  y = database[c(1:pH),1];
  Area3 = trapz(x,y);


  x = database[c(pH:max(descent1)),3];
  y = database[c(pH:max(descent1)),1];
  Area4 = trapz(x,y);
  }

  Springiness = Distance2/Distance1;

  if(Area1!=0){
    Cohesiveness = Area2/Area1;
  } else {
    Cohesiveness = NA;
  }
  if(Area3!=0){
    Resilience = Area4/Area3;
  } else {
    Resilience = NA;
  }

  
  a1 = StrainBounds[1]; #0.075;
  a2 = StrainBounds[2]; #0.15;
  ra1 = which(strain[ascent1,1]>a1)
  ra2 = which(strain[ascent1,1]<a2)
  a = intersect(ra1,ra2);
  fit1 = lm(stress[a]~strain[a])
  # plot(strain[a],stress[a], xlab="strain [mm/mm]",ylab=expression(paste("stress [",N/mm^2,"]")))
  fo = summary(fit1)$coefficients[1];
  EmArun = summary(fit1)$coefficients[2];
  bArun=confint(fit1,2,0.95);
  # abline(fo,EmArun)
  
  # Ouput Table
  Output = as.data.frame(array(0,dim=c(8,2)))
  Output[1,1] = "Hardness (N)";
  Output[1,2] = Hardness1;
  Output[2,1] = "Springiness";
  Output[2,2] = Springiness;
  Output[3,1] = "Cohesiveness";
  Output[3,2] = Cohesiveness;
  Output[4,1] = "Resilience";
  Output[4,2] = Resilience;
  Output[5,1] = "Young's modulus (stress-strain linear approximation)"
  Output[5,2] = EmArun;
  Output[6,1] = "95% Interval (from linear fit) - lower limit";
  Output[6,2] = bArun[1];
  Output[7,1] = "95% Interval (from linear fit) - upper limit";
  Output[7,2] = bArun[2];
  Output[8,1] = "R-squared (from linear fit)";
  Output[8,2] = round(as.numeric(summary(fit1)[8]),4);
  
  write.csv(Output,"asm-TPA-Package-output.csv")
  print(Output) 
  
  return(EmArun);
}